/*****************************************************************************
 * dump-zap.c: support for DVB-T  scan
 *****************************************************************************
 * $Id: dump-zap.c,v 1.5 2006/12/25 16:34:13 estre Exp $
 *****************************************************************************
 * Program under GNU General Public License (see ../../COPYING)
 *****************************************************************************/

/* This code comes from the linuxtv project : http://linuxtv.org 
*  Thanks to the linuxtv developpers
*/

#include <stdio.h>
#include <linux/dvb/frontend.h>
#include "dump-zap.h"
#include "../translation.h"
#include "../channel.h"
#include "../scan_common.h"
#include "../dvbtools.h"

static const char *inv_name [] = {
	"INVERSION_OFF",
	"INVERSION_ON",
	"INVERSION_AUTO"
};

static const char *fec_name [] = {
	"FEC_NONE",
	"FEC_1_2",
	"FEC_2_3",
	"FEC_3_4",
	"FEC_4_5",
	"FEC_5_6",
	"FEC_6_7",
	"FEC_7_8",
	"FEC_8_9",
	"FEC_AUTO"
};


static const char *qam_name [] = {
	"QPSK",
	"QAM_16",
	"QAM_32",
	"QAM_64",
	"QAM_128",
	"QAM_256",
	"QAM_AUTO"
};


static const char *bw_name [] = {
	"BANDWIDTH_8_MHZ",
	"BANDWIDTH_7_MHZ",
	"BANDWIDTH_6_MHZ",
	"BANDWIDTH_AUTO"
};


static const char *mode_name [] = {
	"TRANSMISSION_MODE_2K",
	"TRANSMISSION_MODE_8K",
	"TRANSMISSION_MODE_AUTO"
};

static const char *guard_name [] = {
	"GUARD_INTERVAL_1_32",
	"GUARD_INTERVAL_1_16",
	"GUARD_INTERVAL_1_8",
	"GUARD_INTERVAL_1_4",
	"GUARD_INTERVAL_AUTO"
};


static const char *hierarchy_name [] = {
	"HIERARCHY_NONE",
	"HIERARCHY_1",
	"HIERARCHY_2",
	"HIERARCHY_4",
	"HIERARCHY_AUTO"
};

void xdtv_dump_dvb_parameters (FILE *f, fe_type_t type, struct dvb_frontend_parameters *p, char polarity, int sat_number)
{
	switch (type) {
	case FE_QPSK:
		break;

	case FE_QAM:
		break;

	case FE_OFDM:
		fprintf(f,"frequency = %i\ninversion = %s\nbandwidth = %s\ncode_rate_high = %s\ncode_rate_low = %s\nconstellation = %s\ntransmission_mode = %s\nguard_interval = %s\nhierarchy = %s\n",
		p->frequency ,inv_name[p->inversion],bw_name[p->u.ofdm.bandwidth],fec_name[p->u.ofdm.code_rate_HP],
		fec_name[p->u.ofdm.code_rate_LP],qam_name[p->u.ofdm.constellation],mode_name[p->u.ofdm.transmission_mode],
		guard_name[p->u.ofdm.guard_interval],hierarchy_name[p->u.ofdm.hierarchy_information]);
		break;

	default:
		;
	};
}


void zap_dump_dvb_parameters (FILE *f, fe_type_t type, struct dvb_frontend_parameters *p, char polarity, int sat_number)
{
	switch (type) {
	case FE_QPSK:
		fprintf (f, "%i:", p->frequency / 1000);	/* channels.conf wants MHz */
		fprintf (f, "%c:", polarity);
		fprintf (f, "%d:", sat_number);
		fprintf (f, "%i", p->u.qpsk.symbol_rate / 1000); /* channels.conf wants kBaud */
		/*fprintf (f, "%s", fec_name[p->u.qpsk.fec_inner]);*/
		break;

	case FE_QAM:
		fprintf (f, "%i:", p->frequency);
		fprintf (f, "%s:", inv_name[p->inversion]);
		fprintf (f, "%i:", p->u.qpsk.symbol_rate);
		fprintf (f, "%s:", fec_name[p->u.qpsk.fec_inner]);
		fprintf (f, "%s", qam_name[p->u.qam.modulation]);
		break;

	case FE_OFDM:
		fprintf (f, "%i:", p->frequency);
		fprintf (f, "%s:", inv_name[p->inversion]);
		fprintf (f, "%s:", bw_name[p->u.ofdm.bandwidth]);
		fprintf (f, "%s:", fec_name[p->u.ofdm.code_rate_HP]);
		fprintf (f, "%s:", fec_name[p->u.ofdm.code_rate_LP]);
		fprintf (f, "%s:", qam_name[p->u.ofdm.constellation]);
		fprintf (f, "%s:", mode_name[p->u.ofdm.transmission_mode]);
		fprintf (f, "%s:", guard_name[p->u.ofdm.guard_interval]);
		fprintf (f, "%s", hierarchy_name[p->u.ofdm.hierarchy_information]);
		break;

	default:
		;
	};
}

void xdtv_dump_service_parameter_set (FILE *f, 
				 const char *service_name,
				 fe_type_t type,
				 struct dvb_frontend_parameters *p,
				 char polarity,
				 int sat_number,
				 uint16_t video_pid,
				 uint16_t *audio_pid,
				 uint16_t service_id)
{
	struct CHANNEL* cur_chan=NULL;
	chan_handler_t* chan_handler=get_chan_handler();

	if (video_pid == 0){
		fprintf(stderr,"[%s] : %s\n", service_name, translate("no video, we skip this channel"));
		return ;
	}



	fprintf (f, "[%s]\nchannel = 0\ntype = DVB_T\n", service_name);
	fprintf(f,"frequency = %i\ninversion = %s\nbandwidth = %s\ncode_rate_high = %s\ncode_rate_low = %s\nconstellation = %s\ntransmission_mode = %s\nguard_interval = %s\nhierarchy = %s\n",
		p->frequency ,inv_name[p->inversion],bw_name[p->u.ofdm.bandwidth],fec_name[p->u.ofdm.code_rate_HP],
		fec_name[p->u.ofdm.code_rate_LP],qam_name[p->u.ofdm.constellation],mode_name[p->u.ofdm.transmission_mode],
		guard_name[p->u.ofdm.guard_interval],hierarchy_name[p->u.ofdm.hierarchy_information]);

//	xdtv_dump_dvb_parameters (f, type, p, polarity, sat_number);
	fprintf (f, "vpid = %d\napid = %d\nspid = %d\n\n", video_pid, audio_pid[0], service_id);
	fflush(f);

//	fprintf(stderr,"dump-zap.c : try to lock mutex\n");
	pthread_mutex_lock(&(chan_handler->scan_mutex));
//	fprintf(stderr,"dump-zap.c : ok mutex locked\n");
  	chan_handler->new_channel_available=1;

	  /* channel init */
	  
	chan_handler->chan_number++;
	cur_chan=&(chan_handler->new_channels[chan_handler->chan_number]);
	memset(cur_chan,0,sizeof * cur_chan);
	cur_chan->name=NULL;
	cur_chan->key=NULL;
	cur_chan->cname=NULL;
	cur_chan->subpage=NULL;

	cur_chan->name=strdup(service_name);
	cur_chan->key=strdup("x"); 
  	cur_chan->cname=strdup("xx");
	cur_chan->channel_type = DVB_T ;

	cur_chan->dvb_freq=p->frequency;
	cur_chan->inversion=str_to_int(inv_name[p->inversion],inversion_list);
	cur_chan->bandwidth=str_to_int(bw_name[p->u.ofdm.bandwidth] ,bw_list );
	cur_chan->code_rate_high=str_to_int(fec_name[p->u.ofdm.code_rate_HP] , fec_list);
	cur_chan->code_rate_low=str_to_int(fec_name[p->u.ofdm.code_rate_LP] ,fec_list );
	cur_chan->constellation=str_to_int(qam_name[p->u.ofdm.constellation] , constellation_list);
	cur_chan->transmission_mode=str_to_int(mode_name[p->u.ofdm.transmission_mode] ,transmissionmode_list );
	cur_chan->guard_interval=str_to_int(guard_name[p->u.ofdm.guard_interval] ,guard_list );
	cur_chan->hierarchy=str_to_int(hierarchy_name[p->u.ofdm.hierarchy_information] ,hierarchy_list );
	cur_chan->vpid=video_pid;
	cur_chan->apid=audio_pid[0];
	cur_chan->spid=service_id;

//	fprintf(stderr,"dump-zap.c : unlock mutex\n");
	pthread_mutex_unlock(&(chan_handler->scan_mutex));

}

